from django.test.simple import DjangoTestSuiteRunner
from django.db.models import get_app
from django.conf import settings
import os
import coverage

try:
    TESTCOVERAGE_ACTIVE = settings.TESTCOVERAGE_ACTIVE
except AttributeError:
    TESTCOVERAGE_ACTIVE = True

try:
    TESTCOVERAGE_APPS = settings.TESTCOVERAGE_APPS
except AttributeError:
    TESTCOVERAGE_APPS = ()

class TestCoverageTestRunner(DjangoTestSuiteRunner):

    def setup_test_environment(self, **kwargs):
        self.__coverage = coverage.coverage()
        self.__coverage.start()
        super(TestCoverageTestRunner, self).setup_test_environment(**kwargs)

    def build_suite(self, test_labels, *args, **kwargs):
        suite = super(TestCoverageTestRunner, self).build_suite(test_labels, *args, **kwargs)
        suite.__test_labels = test_labels
        return suite

    def suite_result(self, suite, *args, **kwargs):
        self.__coverage.stop()
        super(TestCoverageTestRunner, self).suite_result(suite, *args, **kwargs)
        test_labels = suite.__test_labels or [import_path.split('.')[-1] for import_path in TESTCOVERAGE_APPS or settings.INSTALLED_APPS]
        paths = []
        for label in test_labels:
            module = get_app(label)
            module_path = os.path.dirname(module.__file__)
            paths += [os.path.join(listing[0], '*.py') for listing in os.walk(module_path, followlinks=True)]
        self.__coverage.report(paths)

if not TESTCOVERAGE_ACTIVE:
    TestCoverageTestRunner = DjangoTestSuiteRunner

